@@ -52,7 +52,8 @@
   (make-local-variable 'comint-use-prompt-regexp)
   (setq comint-dynamic-complete-functions '(M2-dynamic-complete-symbol comint-dynamic-complete-filename)
 	comint-prompt-regexp M2-comint-prompt-regexp
-	comint-use-prompt-regexp t))
+	comint-use-prompt-regexp t)
+  (add-hook 'comint-output-filter-functions 'M2-info-help nil t))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Common definitions
@@ -66,6 +67,7 @@
   (set (make-local-variable 'comment-start-skip) "-- *")
   (set (make-local-variable 'comint-input-autoexpand) nil)
   (set (make-local-variable 'transient-mark-mode) t)
+  (set (make-local-variable 'indent-line-function) 'M2-electric-tab)
   (setq font-lock-defaults '( M2-mode-font-lock-keywords ))
   (setq truncate-lines t)
   (setq case-fold-search nil))
@@ -74,7 +76,6 @@
 
 (define-key M2-mode-map "\177" 'backward-delete-char-untabify)
 (define-key M2-mode-map "\^M" 'M2-newline-and-indent)
-(define-key M2-mode-map "\t" 'M2-electric-tab)
 ;; (define-key M2-mode-map "}" 'M2-electric-right-brace)
 (define-key M2-mode-map ";" 'M2-electric-semi)
 ;; (define-key M2-mode-map "\^Cd" 'M2-find-documentation)
@@ -482,7 +483,6 @@ be sent can be entered, with history."
   (save-excursion
     (when (string-match "-\\* infoHelp: \\(.*\\) \\*-" string)
       (info-other-window (match-string 1 string)))))
-(add-hook 'comint-output-filter-functions 'M2-info-help)
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; M2-mode
